@@ -1,6 +1,6 @@
 
 """
-    Copyright (C) 2017 SunSpec Alliance
+    Copyright (C) 2018 SunSpec Alliance
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -126,7 +126,7 @@ def model_filename_to_id(filename):
     if filename[0:5] == 'smdx_' and filename[-4:] == '.xml':
         try:
             model_id = int(filename[5:-4])
-        except Exception, e:
+        except Exception as e:
             pass
 
     return model_id
