@@ -1,6 +1,6 @@
 
 """
-    Copyright (C) 2017 SunSpec Alliance
+    Copyright (C) 2018 SunSpec Alliance
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -22,6 +22,11 @@
 """
 
 import sunspec.core.util as util
+import sys
+
+# Python 3 compatibility for long()
+if sys.version_info > (3,):
+    long = int
 
 SUNS_BASE_ADDR_DEFAULT = 40000
 SUNS_SUNS_LEN = 2
