@@ -1,6 +1,6 @@
 
 """
-    Copyright (C) 2016 SunSpec Alliance
+    Copyright (C) 2018 SunSpec Alliance
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -23,37 +23,46 @@
 
 import sys
 import os
+import unittest
 
 import sunspec.core.client as client
 import sunspec.core.device as device
 import sunspec.core.util as util
 
-def test_client_device(pathlist=None):
 
-    try:
-        d = client.ClientDevice(client.MAPPED, slave_id=1, name='mbmap_test_device_1.xml', pathlist = pathlist)
+class TestClientDevice(unittest.TestCase):
+    def setUp(self):
+        path = os.path.abspath(__file__)
+        self.pathlist = util.PathList(['.',
+                                       os.path.join(os.path.dirname(path),
+                                                    'devices')])
+
+        device.check_for_models(pathlist=self.pathlist)
+
+    def test_client_device(self):
+        d = client.ClientDevice(client.MAPPED, slave_id=1,
+                                name='mbmap_test_device_1.xml',
+                                pathlist=self.pathlist)
         d.scan()
         d.read_points()
 
         dp = device.Device()
-        dp.from_pics(filename='pics_test_device_1.xml', pathlist=pathlist)
+        dp.from_pics(filename='pics_test_device_1.xml',
+                        pathlist=self.pathlist)
         not_equal = dp.not_equal(d)
         if not_equal:
             raise Exception(not_equal)
 
-    except Exception, e:
-        print '*** Failure test_client_device: %s' % str(e)
-        return False
-    return True
 
-def test_sunspec_client_device_1(pathlist=None):
-
-    try:
-        d = client.SunSpecClientDevice(client.MAPPED, slave_id=1, name='mbmap_test_device_1.xml', pathlist = pathlist)
+    def test_sunspec_client_device_1(self):
+        d = client.SunSpecClientDevice(client.MAPPED, slave_id=1,
+                                        name='mbmap_test_device_1.xml',
+                                        pathlist=self.pathlist)
         d.read()
 
         dp = device.Device()
-        dp.from_pics(filename='pics_test_device_1.xml', pathlist=pathlist)
+        dp.from_pics(filename='pics_test_device_1.xml',
+                        pathlist=self.pathlist)
         not_equal = dp.not_equal(d.device)
         if not_equal:
             raise Exception(not_equal)
@@ -150,15 +159,9 @@ def test_sunspec_client_device_1(pathlist=None):
 
         d.close()
 
-    except Exception, e:
-        print '*** Failure test_sunspec_client_device_1: %s' % str(e)
-        return False
-    return True
 
-def test_sunspec_client_device_3(pathlist=None):
-
-    try:
-        d = client.SunSpecClientDevice(client.MAPPED, slave_id=1, name='mbmap_test_device_3.xml', pathlist = pathlist)
+    def test_sunspec_client_device_3(self):
+        d = client.SunSpecClientDevice(client.MAPPED, slave_id=1, name='mbmap_test_device_3.xml', pathlist=self.pathlist)
 
         # int16 read and write
         d.model_63002.read()
@@ -168,7 +171,6 @@ def test_sunspec_client_device_3(pathlist=None):
             raise Exception("'model_63002.int16_1' point mismatch: %s %s" % (value, expected))
 
         d.model_63002.repeating[1].int16_1 = 333.3
-        print 'writing...'
         d.model_63002.write()
         d.model_63002.read()
         expected = 3333
@@ -183,40 +185,7 @@ def test_sunspec_client_device_3(pathlist=None):
 
         d.close()
 
-    except Exception, e:
-        print '*** Failure test_sunspec_client_device_3: %s' % str(e)
-        return False
-    return True
-
-tests = [
-    test_client_device,
-    test_sunspec_client_device_1,
-    test_sunspec_client_device_3
-]
-
-def test_all(pathlist=None, stop_on_failure=True):
-
-    if pathlist is None:
-        pathlist = util.PathList(['.', os.path.join(os.path.dirname(os.path.abspath(__file__)), 'devices')])
-
-    count_passed = 0
-    count_failed = 0
-    count_run = 0
-
-    for test in tests:
-        count_run += 1
-        if test(pathlist) is True:
-            count_passed += 1
-        else:
-            count_failed += 1
-            if stop_on_failure is True:
-                break
-
-    print 'Test client module: total tests: %d  tests run: %d  tests passed: %d  tests failed: %d' %  (len(tests), count_run, count_passed, count_failed)
-
-    return (count_run, count_passed, count_failed)
 
 if __name__ == "__main__":
 
-    (count_run, count_passed, count_failed) = test_all()
-    sys.exit(count_failed)
+    unittest.main()
